using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace DragDropTreeSample
{
    class DragDropTreeView : TreeView
    {
        /// <summary>
        /// Private member variable to store the source node which is dragged
        /// </summary>
        private TreeNode srcNodeForDrag;

        /// <summary>
        /// ctor
        /// </summary>
        public DragDropTreeView()
        {
            AssignProperties();
        }

        /// <summary>
        /// Assign the default properties needed for the drag drop operation
        /// </summary>
        private void AssignProperties()
        {
            this.AllowDrop = true;
        }

        /// <summary>
        /// Override the OnDragDrop event handle 
        /// </summary>
        /// <param name="e"></param>
        protected override void OnDragDrop(DragEventArgs e)
        {
            Point pt = PointToClient(new Point(e.X, e.Y));
            TreeNode DestinationNode = this.GetNodeAt(pt);

            // Ensure that the list item index is contained in the data.
            if (e.Data.GetDataPresent(typeof(TreeNode)))
            {
                // Perform drag-and-drop, depending upon the effect.
                if (e.Effect == DragDropEffects.Copy)
                {
                    TreeNode tmpTN = (TreeNode)srcNodeForDrag.Clone();
                    DestinationNode.Nodes.Add(tmpTN);
                }
                else if (e.Effect == DragDropEffects.Move)
                {
                    TreeNode tmpTN = (TreeNode)srcNodeForDrag.Clone();
                    DestinationNode.Nodes.Add(tmpTN);
                    srcNodeForDrag.Remove();
                }
            }
        }

        /// <summary>
        /// Override the OnDragOver event handle
        /// </summary>
        /// <param name="e"></param>
        protected override void OnDragOver(DragEventArgs e)
        {
            if ((e.KeyState & 8) == 8 &&
                (e.AllowedEffect & DragDropEffects.Copy) == DragDropEffects.Copy)
            {
                // CTL KeyState for copy.
                e.Effect = DragDropEffects.Copy;
            }
            else if ((e.AllowedEffect & DragDropEffects.Move) == DragDropEffects.Move)
            {
                e.Effect = DragDropEffects.Move;
            }
        }

        /// <summary>
        /// Override the OnItemDrag event handle
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected override void OnItemDrag(ItemDragEventArgs e)
        {
            srcNodeForDrag = (TreeNode)e.Item;
            this.DoDragDrop(e.Item, DragDropEffects.All);
        }
    }
}
